/*
  bitcoind 'blkxxxx.dat' slicer.
  S. Datskovskiy
  written for therealbitcoin.org
 */


#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


#define MAIN_NET_MAGIC 0xD9B4BEF9


void usage(char *prog) {
  fprintf(stderr, "Usage: %s [-c] [in]\n", prog);
  fprintf(stderr, "-c ..... only count blocks\n");
  exit(1);
}


int main(int argc, char *argv[]) {
  FILE *in, *out;
  uint32_t i = 0, n = 0;
  char *buf, *param;
  char outname[128];
  uint8_t extract = 0;

  if ((argc < 2) || (argc > 3)) {
    usage(argv[0]);
  }

  if (!strcmp(argv[1], "-c")) {
    if (argc < 3) usage(argv[0]);
    param = argv[2];
    extract = 0;
  } else {
    param = argv[1];
    extract = 1;
  }

  in = fopen(param, "rb");
  do {
    if (!fread(&i, sizeof(i), 1, in)) goto fin; /* magic */
    if (i == MAIN_NET_MAGIC) { /* not interested in testnet */
      if (!fread(&i, sizeof(i), 1, in)) goto fin; /* size */
      buf = malloc(i);
      if (!fread(buf, i, 1, in)) {
	fprintf(stderr, "Error: incomplete block!\n");
	exit(-1);
      }
      if (extract) {
	printf("Extracting block #%d in archive %s : %d bytes\n", n, param, i);
	snprintf(outname, sizeof(outname), "%s.%d.blk", param, n);
	out = fopen(outname, "wb");
	if (!fwrite(buf, 1, i, out)) {
	  fprintf(stderr, "Error writing block to file %s!\n", outname);
	  exit(-1);
	}
	fclose(out);
      }
      free(buf);
    }
    n++;
  } while (1);
 fin:
  fclose(in);
  if (!extract) printf("%s : %d blocks.\n", param, n);
  return 0;
}
